﻿using Infragistics.Controls.Charts;
using LogDogBL.Output;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace LogPadTester
{
    /// <summary>
    /// Interaction logic for DiagramEditorControl.xaml
    /// </summary>
    public partial class DiagramEditorControl : UserControl
    {
        public DiagramEditorControl()
        {
            InitializeComponent();
        }



        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            CustomizeToolBox();
        }

        public void LoadControls(bool loadFromFile)
        {
            Diagram.ItemsSource = null;

            ObservableCollection<BaseControlAttributes> controls = new ObservableCollection<BaseControlAttributes>();
            if (loadFromFile)
            {
                var c = LogDogUtils.XmlIO<BaseControlAttributes[]>.Load("D:\\test.xml");
                foreach (var item in c)
                {
                    controls.Add(item);
                }
            }

            else
            {
                double y = 50;
                double x = 50;
                for (int i = 0; i < 10; i++)
                {
                    controls.Add(new ShapeControlAttributes() { Name = "ctrl" + i, Position = new Point(x, y), Width = 150, Height = 50, Shape = DiagramShapeType.Rectangle.ToString() });

                    if (i == 5)
                    {
                        controls.Remove(controls[controls.Count - 1]);
                        controls.Add(new ShapeGaugeControlAttributes() { Name = "text44", Position = new Point(x, y), Width = 150, Height = 50 });
                    }
                    if (i == 7)
                        ((ShapeControlAttributes)controls[controls.Count - 1]).Shape = DiagramShapeType.Circle.ToString();


                    x += 160;
                    if (i > 0 && i % 3 == 2) { x = 160; y += 160; }
                }
            }
            Diagram.ItemsSource = controls;

        }




        private void CustomizeToolBox()
        {
            List<string> shapesToKeep = new List<string>();
            shapesToKeep.Add("Rectangle");
            shapesToKeep.Add("Ellipse");
            shapesToKeep.Add("Triangle");
            //    shapesToKeep.Add("Straight");

            //  toolBox.Categories.Clear();
            // clear shapes, only keep the 3 basic shapes:
            List<DiagramToolboxCategory> categoriesToKeep = new List<DiagramToolboxCategory>();

            foreach (var cat in toolBox.Categories)
            {

                int itemIndex = 0;
                while (itemIndex < cat.Items.Count)
                {
                    if (shapesToKeep.Contains(cat.Items[itemIndex].Title))
                    {
                        ++itemIndex;
                        if (!categoriesToKeep.Contains(cat))
                            categoriesToKeep.Add(cat);
                    }
                    else if (cat.Items[itemIndex].Title.Equals("Text"))
                    {
                        cat.Items[itemIndex].Title = "Text Gauge";
                        ++itemIndex;
                        if (!categoriesToKeep.Contains(cat))
                            categoriesToKeep.Add(cat);
                    }
                    else
                        cat.Items.RemoveAt(itemIndex);
                }
            }
            // remove all but the categories that we kepp:
            int index = 0;

            while (index < toolBox.Categories.Count)
            {
                if (categoriesToKeep.Contains(toolBox.Categories[index]))
                    ++index;
                else
                    toolBox.Categories.RemoveAt(index);
            }

            // Add gauge icons:
            DiagramToolboxCategory gaugesCat = new DiagramToolboxCategory() { Name = "Gauges" };
            DiagramToolboxItem item = new DiagramToolboxItem() { Title = "Text Gauge" };

            ItemContainerTemplate icontemplate = (ItemContainerTemplate)Resources["itemContainerTemplate"];
            item.Item = new GaugeDiagramNode() { Width = 50, Height = 50, DisplayTemplate = icontemplate, ShapeType = DiagramShapeType.Chevron, LeftContentOffset = 0, RightContentOffset = 0 };

            gaugesCat.Items.Add(item);

            //toolBox.Categories.Add(gaugesCat);

        }

        private void Diagram_ItemAdded(object sender, DiagramItemAddedEventArgs e)
        {
            // for gauge item, set the template, and user should be able to select the related variable
            if (e.Item.Content != null && "Text".Equals(e.Item.Content))
            {
                e.Item.Fill = Brushes.LightBlue;
                e.Item.Width = 170;
                e.Item.Height = 50;
                e.Item.DisplayTemplate = (DataTemplate)Resources["GaugeNodeTemplate"];
                //((DiagramNode)e.Item). = new Button() { Content = "HELLO" };
                e.Item.Content = new ShapeGaugeControlAttributes() { Name = "<click to select>" };
            }
            else if (e.Item is DiagramNode dn)
            {
                e.Item.Content = new ShapeControlAttributes() { Name = GetNextName(dn.ShapeType), Shape = dn.ShapeType.ToString() };
                e.Item.DisplayTemplate = (DataTemplate)Resources["NodeTemplate"];


            }
            else
            {

            }
        }

        private string GetNextName(DiagramShapeType shapeType)
        {
            int index = 1;
            string name;
            bool nameExists;
            do
            {

                nameExists = false;
                name = shapeType.ToString() + index;
                ++index;
                foreach (var item in Diagram.Items)
                {
                    if (item.Name.Equals(name, StringComparison.OrdinalIgnoreCase) ||
                        (item.Content is BaseControlAttributes bca && bca.Name.Equals(name, StringComparison.OrdinalIgnoreCase)))
                    {
                        nameExists = true;
                        break;
                    }
                }
            } while (nameExists);

            return name;

        }

        private void Diagram_ItemRemoving(object sender, DiagramItemRemovingEventArgs e)
        {

        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            SaveControls();
        }

        private void SaveControls()
        {
            List<BaseControlAttributes> controls = new List<BaseControlAttributes>();

            foreach (var item in Diagram.Items)
            {
                BaseControlAttributes bca = item.Content as BaseControlAttributes;

                // set the properties based on the item:
                if (bca != null)
                {
                    if (item is DiagramNode dn)
                    {
                        bca.Width = (int)dn.Width;
                        bca.Height = (int)dn.Height;
                        bca.Position = dn.Position;

                        // save related properties:


                        if (bca.Width == 0 || bca.Height == 0 || bca.Position.X == 0 || bca.Position.Y == 0)
                        {

                        }

                    }
                    controls.Add(bca);
                }
                else
                {
                    // all should be control-attributes
                }
            }

            LogDogUtils.XmlIO<BaseControlAttributes[]>.Save(controls.ToArray(), "d:\\test.xml");

        }

        private void brnLoad_Click(object sender, RoutedEventArgs e)
        {
            LoadControls(true);
        }


        private void btnEditDiagram_Click(object sender, RoutedEventArgs e)
        {
            // for the selected control, open the state manager to manage the 'state' of the shape:
            if (sender is Button btn)
            {
                if (btn.Tag is ShapeControlAttributes sca)
                {
                    LogDog.Editors.DiagramStateManager manager = new LogDog.Editors.DiagramStateManager();
                    DiagramNode node = LogDog.WPFUtils.FindParent<DiagramNode>(btn);
                    if (node != null)
                        manager.sampleShape.ShapeType = node.ShapeType;

                    manager.SelectedAttributes = sca;

                    manager.ShowDialog();

                    sca.States = manager.States.ToArray();
                    // run the appearance set again
                    if (node != null)
                        LogDog.Editors.DiagramStateManager.SetShapeProperties(node, sca.DefaultApperance, null);
                }
            }
        }


        private void DiagramNode_Loaded(object sender, RoutedEventArgs e)
        {
            if (sender is DiagramNode dn)
            {
                if (dn.Content is ShapeControlAttributes sca)
                {
                    // based on the provided item, set the diagram node appearance:
                    LogDog.Editors.DiagramStateManager.SetShapeProperties(dn, sca.DefaultApperance, null);
                }
            }
        }
    }

    public class GaugeDiagramNode : DiagramNode
    {

    }


}
